@@ -1646,11 +1646,8 @@ void catalog_op(int op, int x, int y)
 	          else {
                      eventmode = MOVE_MODE;
 		     was_preselected = FALSE;
-		     /* This is a hack, but avoids having to define a	*/
-		     /* separate event type for a move following a	*/
-		     /* library selection, just so we can handle	*/
-		     /* "cancel" differently.				*/
-	             areawin->editpart = -1;
+	 	     register_for_undo(XCF_Library_Pop, UNDO_MORE, areawin->topinstance,
+		  		areawin->selectlist, areawin->selects);
 	          }
 #ifdef TCL_WRAPPER
 		  /* fprintf(stderr, "Creating event handler for xctk_drag: "); */
@@ -1658,8 +1655,6 @@ void catalog_op(int op, int x, int y)
                   Tk_CreateEventHandler(areawin->area, PointerMotionMask,
 			(Tk_EventProc *)xctk_drag, NULL);
 #endif
-	 	  register_for_undo(XCF_Library_Pop, UNDO_MORE, areawin->topinstance,
-				areawin->selectlist, areawin->selects);
                   catreturn();
                }
 
@@ -1677,7 +1672,7 @@ void catalog_op(int op, int x, int y)
 		     newselect = allocselect();
 		     *newselect = newinst;
 		     XcSetFunction(GXcopy);
-		     XcSetForeground(SELECTCOLOR);
+		     XcTopSetForeground(SELECTCOLOR);
 		     UDrawObject(*libobj, SINGLE, SELECTCOLOR, NULL);
 		  }
 	       }
@@ -1742,7 +1737,9 @@ void startcatalog(xcWidget w, pointertype libmod, caddr_t nulldata)
       eventmode = CATALOG_MODE;
    }
    else if (eventmode != CATMOVE_MODE) {
-      finish_op(XCF_Cancel, 0, 0);
+      /* Don't know why I put this here---causes xcircuit to redraw	*/
+      /* the schematic view when switching between library pages.	*/
+      // finish_op(XCF_Cancel, 0, 0);
       eventmode = CATALOG_MODE;
    }
 
@@ -1762,9 +1759,13 @@ void startcatalog(xcWidget w, pointertype libmod, caddr_t nulldata)
    topobject->viewscale = areawin->vscale;
    topobject->pcorner = areawin->pcorner;
    areawin->topinstance = xobjs.libtop[libmod];
-   setpage(TRUE);
 
-   transferselects();
+   if (libmod == FONTLIB)
+      setpage(FALSE);
+   else {
+      setpage(TRUE);
+      transferselects();
+   }
 
    /* draw the new screen */
 
