@@ -153,22 +153,23 @@ eparamptr make_new_eparam(char *key)
    newepp->key = (char *)malloc(1 + strlen(key));
    strcpy(newepp->key, key);
    newepp->pdata.refkey = NULL;		/* equivalently, sets pointno=0 */
+   newepp->flags = 0;
 
    return newepp;
 }
 
 /*----------------------------------------------------------------------*/
 /* Determine if a parameter is indirectly referenced.  If so, return	*/
-/* the reference key.  If not, return NULL.				*/
+/* the parameter name.  If not, return NULL.				*/
 /*----------------------------------------------------------------------*/
 
-char *find_indirect_param(objinstptr thisinst, char *key)
+char *find_indirect_param(objinstptr thisinst, char *refkey)
 {
    eparamptr epp;
 
    for (epp = thisinst->passed; epp != NULL; epp = epp->next) {
-      if (!strcmp(epp->key, key))
-	 return epp->pdata.refkey;
+      if ((epp->flags & P_INDIRECT) && !strcmp(epp->pdata.refkey, refkey))
+	 return epp->key;
    }
    return NULL;
 }
@@ -271,7 +272,7 @@ void free_element_param(genericptr thiselem, eparamptr thisepp)
 	 /* stored as an allocated string in pdata.refkey, which needs	*/
 	 /* to be free'd.						*/
 
-	 if (IS_OBJINST(thiselem) && (epp->pdata.refkey != NULL))
+	 if ((epp->flags & P_INDIRECT) && (epp->pdata.refkey != NULL))
 	    free(epp->pdata.refkey);
 
 	 free(epp->key);
@@ -402,7 +403,8 @@ void setparammarks(genericptr thiselem)
       for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
 	 ops = match_param(topobject, epp->key);
 	 if (ops == NULL) continue;	/* error condition */
-	 XcFakeTagCall(xcinterp, 3, "parameter", "make", translateparamtype(ops->which));
+	 XcInternalTagCall(xcinterp, 3, "parameter", "make",
+			translateparamtype(ops->which));
 	 ptest[ops->which] = TRUE;
       }
    }
@@ -413,7 +415,7 @@ void setparammarks(genericptr thiselem)
 
    for (i = P_POSITION_X; i <= P_COLOR; i++)
       if (ptest[i] != TRUE)
-	 XcFakeTagCall(xcinterp, 3, "parameter", "replace", translateparamtype(i));
+	 XcInternalTagCall(xcinterp, 3, "parameter", "replace", translateparamtype(i));
 }
 #else
 
@@ -594,7 +596,7 @@ int epsubstitute(genericptr thiselem, objectptr thisobj, objinstptr pinst,
       else if (ops == NULL)
 	 continue;
 
-      if (IS_OBJINST(thiselem)) {
+      if ((epp->flags & P_INDIRECT) && (epp->pdata.refkey != NULL)) {
 	 key = epp->pdata.refkey;
 	 if (key != NULL) {
 	    objinstptr thisinst;
@@ -828,9 +830,27 @@ int epsubstitute(genericptr thiselem, objectptr thisobj, objinstptr pinst,
 int opsubstitute(objectptr thisobj, objinstptr pinst)
 {
    genericptr *eptr, *pgen, thiselem;
+   stringpart *strptr;
    int retval = -1;
    Boolean needrecalc;	/* for arcs and splines */
 
+   /* Perform expression parameter substitutions on all labels.	*/
+   /* Note that this used to be done on an immediate basis as	*/
+   /* labels were parsed.  The main difference is that only one	*/
+   /* expression parameter can be used per label if it is to	*/
+   /* compute the result of some aspect of the label, such as	*/
+   /* position; this is a tradeoff for much simplified handling	*/
+   /* of expression results, like having to avoid infinite	*/
+   /* recursion in an expression result.			*/
+
+   for (eptr = thisobj->plist; eptr < thisobj->plist + thisobj->parts; eptr++)
+      if ((*eptr)->type == LABEL)
+	 for (strptr = (TOLABEL(eptr))->string; strptr != NULL; strptr =
+			nextstringpartrecompute(strptr, pinst));
+
+   if (thisobj->params == NULL)
+      return -1;			    /* object has no parameters */
+
    for (eptr = thisobj->plist; eptr < thisobj->plist + thisobj->parts; eptr++) {
 
       needrecalc = False;
@@ -870,30 +890,11 @@ int psubstitute(objinstptr thisinst)
 {
    objinstptr pinst;
    objectptr thisobj;
-   genericptr *eptr;
-   stringpart *strptr;
 
    pinst = (thisinst == areawin->topinstance) ? areawin->topinstance : thisinst;
    if (pinst == NULL) return -1;		/* there is no instance */
    thisobj = pinst->thisobject;
 
-   /* Perform expression parameter substitutions on all labels.	*/
-   /* Note that this used to be done on an immediate basis as	*/
-   /* labels were parsed.  The main difference is that only one	*/
-   /* expression parameter can be used per label if it is to	*/
-   /* compute the result of some aspect of the label, such as	*/
-   /* position; this is a tradeoff for much simplified handling	*/
-   /* of expression results, like having to avoid infinite	*/
-   /* recursion in an expression result.			*/
-
-   for (eptr = thisobj->plist; eptr < thisobj->plist + thisobj->parts; eptr++)
-      if ((*eptr)->type == LABEL)
-	 for (strptr = (TOLABEL(eptr))->string; strptr != NULL; strptr =
-			nextstringpartrecompute(strptr, pinst));
-
-   if (thisobj->params == NULL)
-      return -1;			    /* object has no parameters */
-
    return opsubstitute(thisobj, pinst);
 }
 
@@ -1291,10 +1292,11 @@ eparamptr copyeparam(eparamptr cepp, genericptr thiselem)
    eparamptr newepp;
 
    newepp = make_new_eparam(cepp->key);
-   if (IS_OBJINST(thiselem) && (cepp->pdata.refkey != NULL))
+   if ((cepp->flags & P_INDIRECT) && (cepp->pdata.refkey != NULL))
       newepp->pdata.refkey = strdup(cepp->pdata.refkey);
    else
       newepp->pdata.pointno = cepp->pdata.pointno;  /* also covers pathpt[] */
+   newepp->flags = cepp->flags;
    return newepp;
 }
 
@@ -1365,11 +1367,9 @@ void copyparams(objinstptr destinst, objinstptr sourceinst)
 
    for (cops = psource; cops != NULL; cops = cops->next) {
       if ((ips = match_instance_param(destinst, cops->key)) == NULL) {
-//	 if ((cops->type != XC_EXPR) || (destinst == sourceinst)) {
-            newops = copyparameter(cops);
-	    newops->next = destinst->params;
-	    destinst->params = newops;
-//	 }
+         newops = copyparameter(cops);
+	 newops->next = destinst->params;
+	 destinst->params = newops;
       }
       else if ((cops->type == XC_EXPR) && (ips->type != XC_EXPR))
 	 free_instance_param(destinst, ips);
@@ -1693,7 +1693,9 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key, short cycle)
 	    }
 	    break;
          case POLYGON:
-	    if (loccycle == -1) loccycle = TOPOLY(gelem)->cycle;
+	    if (loccycle == -1)
+	       loccycle = (TOPOLY(gelem)->cycle != NULL) ?
+			TOPOLY(gelem)->cycle->number : -1;
 	    switch(mode) {
 	       case P_POSITION_X:
 	          pptr = TOPOLY(gelem)->points + loccycle;
@@ -1715,7 +1717,9 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key, short cycle)
 	    }
 	    break;
          case SPLINE:
-	    if (loccycle == -1) loccycle = TOSPLINE(gelem)->cycle;
+	    if (loccycle == -1)
+	       loccycle = (TOSPLINE(gelem)->cycle != NULL) ?
+			TOSPLINE(gelem)->cycle->number : -1;
 	    switch(mode) {
 	       case P_POSITION_X:
 	          pptr = TOSPLINE(gelem)->ctrl + loccycle;
@@ -1738,12 +1742,13 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key, short cycle)
 	    break;
          case PATH:
 	    if (loccycle == -1 && (mode == P_POSITION_X || mode == P_POSITION_Y)) {
-	       pgen = getsubpart(TOPATH(gelem));
-	       pidx = getsubpartindex(TOPATH(gelem));
+	       pgen = getsubpart(TOPATH(gelem), &pidx);
 	       if (ELEMENTTYPE(pgen) == POLYGON)
-	          loccycle = ((polyptr)pgen)->cycle;
+	          loccycle = (((polyptr)pgen)->cycle != NULL) ?
+			((polyptr)pgen)->cycle->number : -1;
 	       else
-	          loccycle = ((splineptr)pgen)->cycle;
+	          loccycle = (((splineptr)pgen)->cycle != NULL) ?
+			((splineptr)pgen)->cycle->number : -1;
 	    }
 	    else {
 	       Fprintf(stderr, "Can't parameterize a path point from "
