@@ -67,7 +67,6 @@ extern void FillPolygon(Display *, Window, GC, XPoint *, int, int, int);
 /* from elements.c: */
 
 /* element constructor functions */
-extern int getsubpartindex(pathptr);
 extern labelptr new_label(objinstptr, stringpart *, int, int, int);
 extern labelptr new_simple_label(objinstptr, char *, int, int, int);
 extern labelptr new_temporary_label(objectptr, char *, int, int);
@@ -97,7 +96,7 @@ extern void charreport(labelptr);
 extern Boolean labeltext(int, char *);
 extern void textreturn(void);
 extern void rejustify(short);
-extern void finddir(polyptr);
+extern void findconstrained(polyptr);
 extern void reversepoints(XPoint *, short);
 extern void reversefpoints(XfPoint *, short);
 extern void freeparts(short *, short);
@@ -106,19 +105,17 @@ extern void unjoin(void);
 extern labelptr findlabelcopy(labelptr, stringpart *);
 extern Boolean neartest(XPoint *, XPoint *);
 extern void join(void);
-extern genericptr getsubpart(pathptr);
+extern genericptr getsubpart(pathptr, int *);
 
 /* interactive manipulation of elements */
 extern void splinebutton(int, int);
-extern void trackspline(xcWidget, caddr_t, caddr_t);
+extern void trackelement(xcWidget, caddr_t, caddr_t);
 extern void arcbutton(int, int);
 extern void trackarc(xcWidget, caddr_t, caddr_t);
 extern void boxbutton(int, int);
 extern void trackbox(xcWidget, caddr_t, caddr_t);
 extern void trackwire(xcWidget, caddr_t, caddr_t);
 extern void startwire(XPoint *);
-extern void trackpoly(xcWidget, caddr_t, caddr_t);
-extern void trackpath(xcWidget, caddr_t, caddr_t);
 extern void setendpoint(short *, short, XPoint **, XPoint *);
 extern void wire_op(int, int ,int);
 
@@ -152,13 +149,14 @@ extern void zoominrefresh(int, int);
 extern void zoomoutrefresh(int, int);
 extern void panrefresh(u_int, int, int, float);
 extern void checkwarp(XPoint *);
-extern void warparccycle(arcptr);
+extern void warparccycle(arcptr, short);
 extern int checkcycle(genericptr, short);
+extern pointselect *getrefpoint(genericptr, XPoint **);
 extern void copyvirtual(void);
 extern void nextpathcycle(pathptr, short);
-extern void nextpolycycle(polyptr, short);
-extern void nextsplinecycle(splineptr, short);
-extern void nextarccycle(arcptr, short);
+extern void nextpolycycle(polyptr *, short);
+extern void nextsplinecycle(splineptr *, short);
+extern void nextarccycle(arcptr *, short);
 extern void buttonhandler(xcWidget, caddr_t, XButtonEvent *);
 extern void keyhandler(xcWidget, caddr_t, XKeyEvent *);
 extern Boolean compatible_function(int);
@@ -880,6 +878,7 @@ extern void enable_selects(objectptr, short *, int);
 extern void disable_selects(objectptr, short *, int);
 extern void selectfilter(xcWidget, pointertype, caddr_t);
 extern Boolean checkselect(short);
+extern Boolean checkselect_draw(short, Boolean);
 extern void geneasydraw(short, int, objectptr, objinstptr);
 extern void gendrawselected(short *, objectptr, objinstptr);
 extern selection *genselectelement(short, u_char, objectptr, objinstptr);
@@ -887,8 +886,8 @@ extern short *allocselect(void);
 extern void setoptionmenu(void);
 extern int test_insideness(int, int, XPoint *);
 extern Boolean pathselect(genericptr *, short, float);
-extern Boolean areaelement(genericptr *);
-extern Boolean selectarea(objectptr);
+extern Boolean areaelement(genericptr *, Boolean, short);
+extern Boolean selectarea(objectptr, short);
 extern void startdesel(xcWidget, caddr_t, caddr_t);
 extern void deselect(xcWidget, caddr_t, caddr_t);
 extern void draw_normal_selected(objectptr, objinstptr);
@@ -898,12 +897,18 @@ extern void clearselects_noundo(void);
 extern void clearselects(void);
 extern void unselect_all(void);
 extern void select_connected_pins();
-extern void reset_connected_pins();
+extern void reset_cycles();
 extern selection *recurselect(short, u_char, pushlistptr *);
 extern short *recurse_select_element(short, u_char);
 extern void startselect(void);
 extern void trackselarea(void);
 extern void trackrescale(void);
+extern pointselect *addcycle(genericptr *, short, u_char);
+extern void copycycles(pointselect **, pointselect **);
+extern void advancecycle(genericptr *, short);
+extern void removecycle(genericptr *);
+extern Boolean checkforcycles(short *, int);
+extern void makerefcycle(pointselect *, short);
 
 /* from text.c: */
 
@@ -945,7 +950,6 @@ extern int find_ordinal(int, labelptr, objinstptr);
 
 extern short UDrawChar(u_char, short, short, int, int);
 extern void UDrawString(labelptr, int, objinstptr);
-extern void UDrawStringNoX(labelptr, int, objinstptr);
 extern TextExtents ULength(labelptr, objinstptr, float, short, XPoint *);
 extern void undrawtext(labelptr);
 extern void redrawtext(labelptr);
