@@ -102,17 +102,24 @@ void selectfilter(xcWidget w, pointertype value, caddr_t calldata)
 #endif
 }
 
-/*----------------------------------------------------------------------------*/
-/* Look at select stack to see if there are any selects; call select, if not. */
-/*----------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Look at select stack to see if there are any selects; call select,	*/
+/* if not.  If draw_selected is True, then the selected items are drawn	*/
+/* in the select color.	 Otherwise, they are not redrawn.		*/
+/*----------------------------------------------------------------------*/
 
-Boolean checkselect(short value)
+Boolean checkselect_draw(short value, Boolean draw_selected)
 {
-   short *check;
+   short *check, savemode;
  
    value &= areawin->filter;	/* apply the selection filter */
 
-   if (areawin->selects == 0) select_element(value);
+   if (areawin->selects == 0) {
+      savemode = eventmode;
+      if (!draw_selected) eventmode = PENDING_MODE;
+      select_element(value);
+      eventmode = savemode;
+   }
    if (areawin->selects == 0) return False;
    for (check = areawin->selectlist; check < areawin->selectlist +
 	areawin->selects; check++)
@@ -121,6 +128,15 @@ Boolean checkselect(short value)
    else return True;
 }
 
+/*----------------------------------------------------------------------------*/
+/* Look at select stack to see if there are any selects; call select, if not. */
+/*----------------------------------------------------------------------------*/
+
+Boolean checkselect(short value)
+{
+    return checkselect_draw(value, False);
+}
+
 /*--------------------------------------------------------------*/
 /* Select list numbering revision when an element is deleted	*/
 /* from an object. 						*/
@@ -173,17 +189,23 @@ void geneasydraw(short instance, int mode, objectptr curobj, objinstptr curinst)
 
 void gendrawselected(short *newselect, objectptr curobj, objinstptr curinst)
 {
-   SetFunction(dpy, areawin->gc, GXcopy);
-   SetForeground(dpy, areawin->gc, BACKGROUND);
+   /* Don't draw selection color when selecting for edit */
+   if (eventmode == PENDING_MODE) return;
+
+   XcSetFunction(GXcopy);
+   XcSetForeground(BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
    indicateparams(*(curobj->plist + *newselect));
 
-   SetFunction(dpy, areawin->gc, GXxor);
-   SetForeground(dpy, areawin->gc, SELECTCOLOR ^ BACKGROUND);
+   XcSetFunction(GXxor);
+   XcSetForeground(SELECTCOLOR ^ BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
 
    SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
    indicateparams(*(curobj->plist + *newselect));
+
+   SetForeground(dpy, areawin->gc, areawin->gccolor);
+   SetFunction(dpy, areawin->gc, areawin->gctype);
 }
 
 /*---------------------------------------------------*/
@@ -374,6 +396,319 @@ Boolean pathselect(genericptr *curgen, short class, float range)
    return selected;
 }
 
+/*------------------------------------------------------*/
+/* Check to see if any selection has registered cycles	*/
+/*------------------------------------------------------*/
+
+Boolean  checkforcycles(short *selectlist, int selects)
+{
+   genericptr pgen;
+   pointselect *cycptr;
+   short *ssel;
+
+   for (ssel = selectlist; ssel < selectlist + selects; ssel++) {
+      pgen = SELTOGENERIC(ssel);
+      switch(pgen->type) {
+         case POLYGON:
+	    cycptr = ((polyptr)pgen)->cycle;
+	    break;
+         case ARC:
+	    cycptr = ((arcptr)pgen)->cycle;
+	    break;
+         case SPLINE:
+	    cycptr = ((splineptr)pgen)->cycle;
+	    break;
+         case LABEL:
+	    cycptr = ((labelptr)pgen)->cycle;
+	    break;
+      }
+      if (cycptr != NULL)
+	 if (cycptr->number != -1)
+	    return True;
+   }
+   return False;
+}
+
+/*--------------------------------------------------------------*/
+/* Copy a cycle selection list from one element to another	*/
+/*--------------------------------------------------------------*/
+
+void copycycles(pointselect **new, pointselect **old)
+{
+   pointselect *pptr;
+   short cycles = 0;
+
+   if (*old == NULL) {
+      *new = NULL;
+      return;
+   }
+
+   for (pptr = *old; !(pptr->flags & LASTENTRY); pptr++, cycles++);
+   cycles += 2;
+   *new = (pointselect *)malloc(cycles * sizeof(pointselect));
+   memcpy(*new, *old, cycles * sizeof(pointselect));
+}
+
+/*--------------------------------------------------------------*/
+/* Create a selection record of selected points in an element.	*/
+/* If a record already exists, and "cycle" is not already in	*/
+/* the list, add it.						*/
+/* "flags" may be set to EDITX or EDITY.  If "flags" is zero,	*/
+/* then flags = EDITX | EDITY is assumed.			*/
+/*--------------------------------------------------------------*/
+
+pointselect *addcycle(genericptr *pgen, short cycle, u_char flags)
+{
+   polyptr ppoly;
+   arcptr parc;
+   splineptr pspline;
+   labelptr plabel;
+   pointselect *pptr, **cycptr;
+   short cycles = 0;
+
+   switch((*pgen)->type) {
+      case POLYGON:
+	 ppoly = TOPOLY(pgen);
+	 cycptr = &ppoly->cycle;
+	 break;
+      case ARC:
+	 parc = TOARC(pgen);
+	 cycptr = &parc->cycle;
+	 break;
+      case SPLINE:
+	 pspline = TOSPLINE(pgen);
+	 cycptr = &pspline->cycle;
+	 break;
+      case LABEL:
+         plabel = TOLABEL(pgen);
+	 cycptr = &plabel->cycle;
+	 break;
+   }
+
+   switch((*pgen)->type) {
+      case POLYGON:
+      case ARC:
+      case SPLINE:
+      case LABEL:		// To-do:  Handle labels separately
+
+	 if (*cycptr == NULL) {
+	    *cycptr = (pointselect *)malloc(sizeof(pointselect));
+	    pptr = *cycptr;
+	    pptr->number = cycle;
+	    pptr->flags = (flags == 0) ? EDITX | EDITY : flags;
+	    pptr->flags |= LASTENTRY;
+	 }
+	 else {
+	    for (pptr = *cycptr; !(pptr->flags & LASTENTRY); pptr++, cycles++) {
+	       if (pptr->number == cycle)
+		  break;
+	       pptr->flags &= ~LASTENTRY;
+	    }
+	    if (pptr->number != cycle) {
+	       pptr->flags &= ~LASTENTRY;
+	       *cycptr = (pointselect *)realloc(*cycptr,
+			(cycles + 2) * sizeof(pointselect));
+	       pptr = *cycptr + cycles + 1;
+	       pptr->number = cycle;
+	       pptr->flags = (flags == 0) ? EDITX | EDITY : flags;
+	       pptr->flags |= LASTENTRY;
+	    }
+	    else {
+	       pptr->flags |= (flags == 0) ? EDITX | EDITY : flags;
+	    }
+	 }
+         break;
+   }
+   return pptr;
+}
+
+/*--------------------------------------------------------------*/
+/* Find the cycle numbered "cycle", and mark it as the		*/
+/* reference point.						*/
+/*--------------------------------------------------------------*/
+
+void makerefcycle(pointselect *cycptr, short cycle)
+{
+   pointselect *pptr, *sptr;
+
+   for (pptr = cycptr;; pptr++) {
+      if (pptr->flags & REFERENCE) {
+	 pptr->flags &= ~REFERENCE;
+	 break;
+      }
+      if (pptr->flags & LASTENTRY) break;
+   }
+
+   for (sptr = cycptr;; sptr++) {
+      if (sptr->number == cycle) {
+	 sptr->flags |= REFERENCE;
+	 break;
+      }
+      if (sptr->flags & LASTENTRY) break;
+   }
+
+   /* If something went wrong, revert to the original reference */
+
+   if (!(sptr->flags & REFERENCE)) {
+      pptr->flags |= REFERENCE;
+   }
+}
+      
+/* Original routine, used 1st entry as reference (deprecated) */
+
+void makefirstcycle(pointselect *cycptr, short cycle)
+{
+   pointselect *pptr, tmpp;
+
+   for (pptr = cycptr;; pptr++) {
+      if (pptr->number == cycle) {
+	 /* swap with first entry */
+	 tmpp = *cycptr;
+	 *cycptr = *pptr;
+	 *pptr = tmpp;
+	 if (cycptr->flags & LASTENTRY) {
+	    cycptr->flags &= ~LASTENTRY;
+	    pptr->flags |= LASTENTRY;
+	 }
+	 return;
+      }
+      if (pptr->flags & LASTENTRY) break;
+   }
+}
+
+/*------------------------------------------------------------------------------*/
+/* Advance a cycle (point) selection from value "cycle" to value "newvalue"	*/
+/* If "newvalue" is < 0 then remove the cycle.					*/
+/*										*/
+/* If there is only one cycle point on the element, then advance its point	*/
+/* number.  If there are multiple points on the element, then change the	*/
+/* reference point by moving the last item in the list to the front.		*/
+/*------------------------------------------------------------------------------*/
+
+void advancecycle(genericptr *pgen, short newvalue)
+{
+   polyptr ppoly;
+   arcptr parc;
+   splineptr pspline;
+   labelptr plabel;
+   pointselect *pptr, *endptr, *fcycle, **cycptr, tmpcyc;
+   short cycles = 0;
+
+   if (newvalue < 0) {
+      removecycle(pgen);
+      return;
+   }
+
+   switch((*pgen)->type) {
+      case POLYGON:
+	 ppoly = TOPOLY(pgen);
+	 cycptr = &ppoly->cycle;
+	 break;
+      case ARC:
+	 parc = TOARC(pgen);
+	 cycptr = &parc->cycle;
+	 break;
+      case SPLINE:
+	 pspline = TOSPLINE(pgen);
+	 cycptr = &pspline->cycle;
+	 break;
+      case LABEL:
+         plabel = TOLABEL(pgen);
+	 cycptr = &plabel->cycle;
+	 break;
+   }
+   if (*cycptr == NULL) return;
+
+   /* Remove any cycles that have only X or Y flags set.	*/
+   /* "Remove" them by shuffling them to the end of the list,	*/
+   /* and marking the one in front as the last entry.		*/
+
+   for (endptr = *cycptr; !(endptr->flags & LASTENTRY); endptr++);
+   pptr = *cycptr;
+   while (pptr < endptr) {
+      if ((pptr->flags & (EDITX | EDITY)) != (EDITX | EDITY)) {
+	 tmpcyc = *endptr; 
+	 *endptr = *pptr;
+	 *pptr = tmpcyc;
+	 pptr->flags &= ~LASTENTRY;
+	 pptr->number = -1;
+	 endptr--;
+	 endptr->flags |= LASTENTRY;
+      }
+      else
+	 pptr++;
+   }
+
+   if (pptr->flags & LASTENTRY) {
+      if ((pptr->flags & (EDITX | EDITY)) != (EDITX | EDITY)) {
+	 pptr->flags &= ~LASTENTRY;
+	 pptr->number = -1;
+	 endptr--;
+	 endptr->flags |= LASTENTRY;
+      }
+   }
+
+   /* Now advance the cycle */
+
+   pptr = *cycptr;
+   if (pptr->flags & LASTENTRY) {
+      pptr->number = newvalue;
+   }
+   else {
+      fcycle = *cycptr;
+      for (pptr = fcycle + 1;; pptr++) {
+	 if (pptr->flags & (EDITX | EDITY))
+	    fcycle = pptr;
+	 if (pptr->flags & LASTENTRY) break;
+      }
+      makerefcycle(*cycptr, fcycle->number);
+   }
+}
+
+/*--------------------------------------*/
+/* Remove a cycle (point) selection	*/
+/*--------------------------------------*/
+
+void removecycle(genericptr *pgen)
+{
+   polyptr ppoly;
+   pathptr ppath;
+   arcptr parc;
+   splineptr pspline;
+   labelptr plabel;
+   pointselect *pptr, **cycptr = NULL;
+   genericptr *pathgen;
+
+   switch((*pgen)->type) {
+      case POLYGON:
+	 ppoly = TOPOLY(pgen);
+	 cycptr = &ppoly->cycle;
+	 break;
+      case ARC:
+	 parc = TOARC(pgen);
+	 cycptr = &parc->cycle;
+	 break;
+      case SPLINE:
+	 pspline = TOSPLINE(pgen);
+	 cycptr = &pspline->cycle;
+	 break;
+      case LABEL:
+         plabel = TOLABEL(pgen);
+	 cycptr = &plabel->cycle;
+	 break;
+      case PATH:
+         ppath = TOPATH(pgen);
+	 for (pathgen = ppath->plist; pathgen < ppath->plist + ppath->parts;
+			pathgen++)
+	    removecycle(pathgen);
+	 break;
+   }
+   if (cycptr == NULL) return;
+   if (*cycptr == NULL) return;
+   free(*cycptr);
+   *cycptr = NULL;
+}
+
 /*--------------------------------------*/
 /* Select one of the elements on-screen */
 /*--------------------------------------*/
@@ -450,23 +785,11 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 	 genericptr *pathp;
 
 	 /* Accept path if any subcomponent of the path is accepted */
-	 /* Record which part was selected by setting its cycle to zero */
 
  	 for (pathp = TOPATH(curgen)->plist; pathp < TOPATH(curgen)->plist +
 		TOPATH(curgen)->parts; pathp++)
 	    if (pathselect(pathp, SPLINE|ARC|POLYGON, range)) {
 	       selected = True;
-	       switch((*pathp)->type) {
-	          case POLYGON:
-		     TOPOLY(pathp)->cycle = 0;
-		     break;
-	          case ARC:
-		     TOARC(pathp)->cycle = 0;
-		     break;
-	          case SPLINE:
-		     TOSPLINE(pathp)->cycle = 0;
-		     break;
-	       }
 	       break;
 	    }
       }
@@ -512,10 +835,11 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 /* select arc, curve, and polygon objects from a defined box area */
 /*----------------------------------------------------------------*/
 
-Boolean areaelement(genericptr *curgen)
+Boolean areaelement(genericptr *curgen, Boolean is_path, short level)
 {
    Boolean selected;
    pointlist    currentpt;
+   short	cycle;
 
    switch(ELEMENTTYPE(*curgen)) {
 
@@ -532,28 +856,38 @@ Boolean areaelement(genericptr *curgen)
 	   /* check each point of the polygons */
 
 	   selected = False;
+	   cycle = 0;
            for (currentpt = TOPOLY(curgen)->points; currentpt <
-		TOPOLY(curgen)->points + TOPOLY(curgen)->number; currentpt++) {
+		TOPOLY(curgen)->points + TOPOLY(curgen)->number;
+		currentpt++, cycle++) {
               if ((currentpt->x < areawin->save.x) && (currentpt->x >
 	           areawin->origin.x) && (currentpt->y < areawin->save.y) &&
 	           (currentpt->y > areawin->origin.y)) {
 	         selected = True;
-		 break;
+		 if (level == 0) addcycle(curgen, cycle, 0);
 	      }
-           }
+	   }
 	   break;
 
       case(SPLINE):
 	   /* check each control point of the spline */
 	
-           selected = ((TOSPLINE(curgen)->ctrl[0].x < areawin->save.x) && 
+	   selected = False;
+           if ((TOSPLINE(curgen)->ctrl[0].x < areawin->save.x) && 
 	   	(TOSPLINE(curgen)->ctrl[0].x > areawin->origin.x) &&
 		(TOSPLINE(curgen)->ctrl[0].y < areawin->save.y) &&
-		(TOSPLINE(curgen)->ctrl[0].y > areawin->origin.y))
-		|| ((TOSPLINE(curgen)->ctrl[3].x < areawin->save.x) &&
+		(TOSPLINE(curgen)->ctrl[0].y > areawin->origin.y)) {
+	      selected = True;
+	      if (level == 0) addcycle(curgen, 0, 0);
+	   }
+
+	   if ((TOSPLINE(curgen)->ctrl[3].x < areawin->save.x) &&
 		(TOSPLINE(curgen)->ctrl[3].x > areawin->origin.x) &&
 		(TOSPLINE(curgen)->ctrl[3].y < areawin->save.y) &&
-		(TOSPLINE(curgen)->ctrl[3].y > areawin->origin.y));
+		(TOSPLINE(curgen)->ctrl[3].y > areawin->origin.y)) {
+	      selected = True;
+	      if (level == 0) addcycle(curgen, 3, 0);
+	   }
 	   break;
    }
    return selected;
@@ -563,7 +897,7 @@ Boolean areaelement(genericptr *curgen)
 /* select all objects from a defined box area */
 /*--------------------------------------------*/
 
-Boolean selectarea(objectptr selobj)
+Boolean selectarea(objectptr selobj, short level)
 {
    short	*newselect;
    genericptr   *curgen, *pathgen;
@@ -619,7 +953,7 @@ Boolean selectarea(objectptr selobj)
             UPushCTM();
             UPreMultCTM(DCTM, curinst->position, curinst->scale,
 			curinst->rotation);
-            selected = selectarea(curinst->thisobject);
+            selected = selectarea(curinst->thisobject, level + 1);
             UPopCTM();
             areawin->save.x = savesave.x;
             areawin->save.y = savesave.y;
@@ -713,12 +1047,12 @@ Boolean selectarea(objectptr selobj)
 	   selected = False;
 	   for (pathgen = TOPATH(curgen)->plist; pathgen < TOPATH(curgen)->plist
 		  + TOPATH(curgen)->parts; pathgen++) {
-	      if (areaelement(pathgen)) selected = True;
+	      if (areaelement(pathgen, True, level)) selected = True;
 	   }
 	   break;
 
 	default:
-	   selected = areaelement(curgen);
+	   selected = areaelement(curgen, False, level);
 	   break;
       }
 
@@ -789,6 +1123,7 @@ void draw_all_selected()
    int j;
 
    if (areawin->hierstack != NULL) return;
+
    for (j = 0; j < areawin->selects; j++)
       gendrawselected(areawin->selectlist + j, topobject, areawin->topinstance);
 }
@@ -858,7 +1193,7 @@ void freeselects()
 void clearselects_noundo()
 {
    if (areawin->selects > 0) {
-      if (areawin->multiedit != True) reset_connected_pins();
+      reset_cycles();
       freeselects();
       if (xobjs.suspend < 0) {
          setallstylemarks(areawin->style);
@@ -1146,7 +1481,7 @@ void label_connect_cycles(labelptr thislab)
 			cpoly->number; testpt++) {
 		  if (testpt->x == thislab->position.x
 			&& testpt->y == thislab->position.y) {
-		     cpoly->cycle = cycle;
+		     addcycle(pgen, cycle, 0);
 		     break;
 		  }
 		  else
@@ -1155,7 +1490,7 @@ void label_connect_cycles(labelptr thislab)
 	    }
 	    else {
 	       /* Make sure that this polygon's cycle is not set! */
-	       cpoly->cycle = -1;
+	       removecycle(pgen);
 	    }
 	 }
       }
@@ -1199,7 +1534,7 @@ void inst_connect_cycles(objinstptr thisinst)
 		     for (testpt = cpoly->points; testpt < cpoly->points +
 				cpoly->number; testpt++) {
 		        if (testpt->x == refpoint.x && testpt->y == refpoint.y) {
-			   cpoly->cycle = cycle;
+			   addcycle(pgen, cycle, 0);
 			   break;
 		        }
 		        else
@@ -1208,7 +1543,7 @@ void inst_connect_cycles(objinstptr thisinst)
 		  }
 	          else {
 		     /* Make sure that this polygon's cycle is not set! */
-		     cpoly->cycle = -1;
+		     removecycle(pgen);
 		  }
 	       }
 	    }
@@ -1249,18 +1584,14 @@ void select_connected_pins()
 /* connected to pins of an object instance).				*/
 /*----------------------------------------------------------------------*/
 
-void reset_connected_pins()
+void reset_cycles()
 {
    polyptr cpoly;
    genericptr *pgen;
 
    for (pgen = topobject->plist; pgen < topobject->plist +
-			topobject->parts; pgen++) {
-      if (ELEMENTTYPE(*pgen) == POLYGON) {
-	 cpoly = TOPOLY(pgen);
-	 cpoly->cycle = -1;
-      }
-   }
+			topobject->parts; pgen++)
+      removecycle(pgen);
 }
 
 /*----------------------------------------------------------------------*/
