@@ -46,10 +46,11 @@
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   float angle1;
-   float angle2;
-   short radius;
-   short yaxis;
+   float  angle1;
+   float  angle2;
+   short  radius;
+   short  yaxis;
+   XPoint position;
 } arcinfo;
 
 /* Smaller data structure used when saving paths in the undo stacks.    */
@@ -59,7 +60,6 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;
-   short	cycle;
    u_short	style;
    float	width;
    XPoint	ctrl[4];
@@ -77,6 +77,7 @@ typedef struct editelem {
       pointlist points;		/* original polygon */
       arcinfo *arcspecs;	/* original arc values */
       pathinfo *pathspecs;	/* original parts of a path */
+      XPoint	instpos;	/* original position, for an object instance */ 
    } save;
 } editelement;
 
@@ -158,7 +159,8 @@ short *regen_selection(objinstptr thisinst, uselection *srec)
    Boolean reorder = False;
    short *slist;
 
-   slist = (short *)malloc(srec->number * sizeof(short));
+   if (srec->number > 0)
+      slist = (short *)malloc(srec->number * sizeof(short));
 
    k = 0;
    for (i = 0; i < srec->number; i++) {
@@ -191,7 +193,7 @@ short *regen_selection(objinstptr thisinst, uselection *srec)
    }
 
    if (k == 0) {
-      free(slist);
+      if (srec->number > 0) free(slist);
       return NULL;
    }
    else
@@ -287,7 +289,7 @@ int select_previous(Undoptr thisrecord)
    Undoptr chkrecord;
    uselection *srec;
 
-   unselect_all();
+   clearselects_noundo();
    for (chkrecord = thisrecord->next; chkrecord != NULL; chkrecord = chkrecord->next) {
 
       /* Selections may cross page changes, but only if in the same series */
@@ -516,12 +518,16 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	       erec->save.points =
 			copypoints((pointlist)((splineptr)egen)->ctrl, 4);
 	       break;
+	    case OBJINST:
+	       erec->save.instpos = ((objinstptr)egen)->position;
+	       break;
 	    case ARC:
 	       erec->save.arcspecs = (arcinfo *)malloc(sizeof(arcinfo));
 	       erec->save.arcspecs->angle1 = ((arcptr)egen)->angle1;
 	       erec->save.arcspecs->angle2 = ((arcptr)egen)->angle2;
 	       erec->save.arcspecs->radius = ((arcptr)egen)->radius;
 	       erec->save.arcspecs->yaxis = ((arcptr)egen)->yaxis;
+	       erec->save.arcspecs->position = ((arcptr)egen)->position;
 	       break;
 	    case PATH:
 	       newrecord->idata = ((pathptr)egen)->parts;  /* not needed? */
@@ -637,6 +643,7 @@ short undo_one_action()
    arcptr thisarc;
    splineptr thisspline;
    graphicptr thisgraphic;
+   Boolean need_redraw;
    XCWindowData *savewindow = areawin;
 
    /* Undo the recorded action and shift the undo record pointer.	*/
@@ -736,8 +743,12 @@ short undo_one_action()
 	 /* If there was a previous selection in the undo list,	*/
 	 /* revert to it.					*/
 	 
+	 need_redraw = (areawin->selects > 0) ? True : False;
 	 select_previous(thisrecord);
-	 draw_all_selected();
+	 if (need_redraw)
+	    drawarea(areawin->area, NULL, NULL);
+	 else
+	    draw_all_selected();
 	 break;
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
@@ -780,14 +791,24 @@ short undo_one_action()
 	       tmpinfo.angle2 = earc->angle2; 
 	       tmpinfo.radius = earc->radius; 
 	       tmpinfo.yaxis = earc->yaxis; 
+	       tmpinfo.position = earc->position; 
 	       earc->angle1 = erec->save.arcspecs->angle1;
 	       earc->angle2 = erec->save.arcspecs->angle2;
 	       earc->radius = erec->save.arcspecs->radius;
 	       earc->yaxis = erec->save.arcspecs->yaxis;
+	       earc->position = erec->save.arcspecs->position;
 	       *(erec->save.arcspecs) = tmpinfo;
 	       calcarc(earc);
 	       drawarea(areawin->area, NULL, NULL);
-	    }
+	    } break;
+	    case OBJINST: {
+	       XPoint tmppt;
+	       objinstptr einst = (objinstptr)(erec->element);
+	       tmppt = einst->position;
+	       einst->position = erec->save.instpos;
+	       erec->save.instpos = ((objinstptr)egen)->position;
+	       drawarea(areawin->area, NULL, NULL);
+	    }  break;
 	    case POLYGON: {
 	       pointlist tmppts;
 	       int tmpnum;
@@ -960,7 +981,7 @@ short undo_one_action()
 	 delta = (XPoint *)thisrecord->undodata;
 	 select_connected_pins();
 	 placeselects(-(delta->x), -(delta->y), NULL);
-	 reset_connected_pins();
+	 reset_cycles();
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1163,6 +1184,14 @@ short redo_one_action()
 	       resolveparams(thisrecord->thisinst);
 	       redrawtext(elab);
 	    } break;
+	    case OBJINST: {
+	       XPoint tmppt;
+	       objinstptr einst = (objinstptr)(erec->element);
+	       tmppt = einst->position;
+	       einst->position = erec->save.instpos;
+	       erec->save.instpos = ((objinstptr)egen)->position;
+	       drawarea(areawin->area, NULL, NULL);
+	    }  break;
 	    case ARC: {
 	       arcinfo tmpinfo;
 	       arcptr earc = (arcptr)(erec->element);
@@ -1170,14 +1199,16 @@ short redo_one_action()
 	       tmpinfo.angle2 = earc->angle2; 
 	       tmpinfo.radius = earc->radius; 
 	       tmpinfo.yaxis = earc->yaxis; 
+	       tmpinfo.position = earc->position; 
 	       earc->angle1 = erec->save.arcspecs->angle1;
 	       earc->angle2 = erec->save.arcspecs->angle2;
 	       earc->radius = erec->save.arcspecs->radius;
 	       earc->yaxis = erec->save.arcspecs->yaxis;
+	       earc->position = erec->save.arcspecs->position;
 	       *(erec->save.arcspecs) = tmpinfo;
 	       calcarc(earc);
 	       drawarea(areawin->area, NULL, NULL);
-	    }
+	    } break;
 	    case POLYGON: {
 	       pointlist tmppts;
 	       int tmpnum;
@@ -1328,7 +1359,7 @@ short redo_one_action()
 	 delta = (XPoint *)thisrecord->undodata;
 	 select_connected_pins();
 	 placeselects(delta->x, delta->y, NULL);
-	 reset_connected_pins();
+	 reset_cycles();
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
