@@ -147,7 +147,7 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 	((a)->parts + 1) * sizeof(genericptr))
 
 #define ENDPART		topobject->plist + topobject->parts
-#define EDITPART	topobject->plist + areawin->editpart
+#define EDITPART	topobject->plist + *areawin->selectlist
 
 /* Type checks (argument "a" is type genericptr) */
 
@@ -306,6 +306,7 @@ enum editmode {
   COPY_MODE,		/* In the process of copying elements */
   PAN_MODE,		/* In the process of panning to follow the cursor */
   SELAREA_MODE,		/* Area selection box */
+  PENDING_MODE,		/* Temporary mode to select without drawing selection */
   RESCALE_MODE,		/* Interactive element rescaling box */
   CATALOG_MODE,		/* On a library page, library directory, or page directory */
   CATTEXT_MODE,		/* Editing an existing object name in the library */
@@ -446,16 +447,21 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 /* Box edit styles							*/
 /*----------------------------------------------------------------------*/
 
+#define NONE  0
 #define MANHATTAN 1
 #define RHOMBOIDX 2
 #define RHOMBOIDY 4
 #define RHOMBOIDA 8
 
-#define NONE  0
-#define LASTX 1
-#define NEXTX 2
-#define LASTY 4
-#define NEXTY 8
+/*----------------------------------------------------------------------*/
+/* Cycle points (selected points) (flag bits, can be OR'd together)	*/
+/*----------------------------------------------------------------------*/
+
+#define EDITX 	  0x01
+#define EDITY	  0x02
+#define LASTENTRY 0x04
+#define PROCESS	  0x08
+#define REFERENCE 0x10
 
 /*----------------------------------------------------------------------*/
 /* Arc creation and edit styles						*/
@@ -651,6 +657,11 @@ typedef struct {
    short *idx;
 } uselection;
 
+typedef struct {
+   short number;
+   u_char flags;
+} pointselect;
+
 /*----------------------------------------------------------------------*/
 /* Bounding box								*/
 /*----------------------------------------------------------------------*/
@@ -737,7 +748,7 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   short	cycle;		/* Edit position, or -1	*/
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    XPoint	position;
    short	rotation;
    float	scale;
@@ -754,7 +765,7 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   short	cycle;		/* Edit position, or -1	*/
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    u_short	style;
    float	width;
    short	number;
@@ -769,7 +780,7 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   short	cycle;		/* Edit position, or -1	*/
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    u_short	style;
    float	width;
    XPoint	ctrl[4];
@@ -785,7 +796,7 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   short	cycle;		/* Edit position, or -1	*/
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    u_short	style;
    float	width;
    short	radius; 	/* x-axis radius */
@@ -1155,9 +1166,12 @@ typedef struct {
 #define XcSetFunction(z) SetFunction(dpy, areawin->gc, z); \
 	areawin->gctype = z
 
-#define XcSetForeground(z) XTopSetForeground(z); areawin->gccolor = \
+#define XcTopSetForeground(z) XTopSetForeground(z); areawin->gccolor = \
 	((z) == DEFAULTCOLOR) ? FOREGROUND : (z)
 
+#define XcSetForeground(z) SetForeground(dpy, areawin->gc, z); \
+	areawin->gccolor = z
+
 #define XcSetXORFg(y,z) XSetXORFg(y,z); areawin->gccolor = \
 	((y) == DEFAULTCOLOR) ? (FOREGROUND ^ z) : (y ^ z)
 
@@ -1369,9 +1383,7 @@ typedef struct _windowdata {
    XPoint	save, origin;
    short	selects;
    short	*selectlist;
-   short	editpart;
    short	attachto;
-   Boolean	multiedit;
    short	lastlibrary;
    short	textpos;
    short	textend;
